require("hump.class")
require("LUBE")
require("unicode")

function love.load()
	font = love.graphics.newFont("consola.ttf", 16)
	love.graphics.setFont(font)
	display = ""
	line = ""
	mode = "none"
	addline("Type /server or /client")
end
	
function love.update(dt)
	if mode == "client" or mode == "server" then
		conn:update(dt)
	end
end

function love.draw()
	love.graphics.setColor(100, 100, 100)
	love.graphics.rectangle("fill", 0, 0, 640, 20)
	love.graphics.setColor(255, 255, 255)
	love.graphics.print(line, 0, 0)
	love.graphics.print(display, 0, 20)
end

function love.keypressed(key, unicode)
	
	if key == "escape" then
		love.event.push("quit")
	end
	
	if key == "return" then
		if line == "/server" then
			listen(7777)
		elseif line =="/client" then
			connect("localhost", 7777)
		elseif line =="/mode" then
			addline("The current mode is: " .. mode)
		elseif line == "/exit" or line =="/quit" then
			love.event.push("quit")
		else
			if mode == "server" then
				line = "SERVER: " .. line
				addline(line)
				conn:send(line)
			elseif mode == "client" then
				conn:send(line)
			end
		end
		line = ""		
	end
	
	if key == "backspace" then
		line = line:sub(1, #line - 1)
	end
	
	-- ascii
	if unicode >= 32 and unicode <= 126 then
        line = line .. string.char(unicode)
    end	
	
	-- unicode
	if unicode >= 0x00A0 and unicode <= 0x024F then
		line = line .. CodeToUTF8(unicode)
	end	
end

-- start server
function listen(port)
	conn = lube.udpServer()
	conn.handshake = "password"
	conn:listen(port)
	conn.callbacks.connect = call_connect
	conn.callbacks.disconnect = call_disconnect
	conn.callbacks.recv = call_recv
	addline("UDP server listening on port " .. tostring(port))	
	mode = "server"
end

-- connect client
function connect(address, port)
	conn = lube.udpClient()
	conn.handshake = "password"
	assert(conn:connect(address, port))
	conn.callbacks.recv = call_recv_client
	addline("Sending UDP data to " .. address .. ":" .. tostring(port))
	mode = "client"
end

-- callbacks
function call_connect(clientid)
	local msg = "SERVER: New connection from " .. clientid
	conn:send(msg)
	addline(msg)
end

function call_disconnect(clientid)
	local msg = "SERVER: Disconnected client " .. clientid
	conn:send(msg)
	addline(msg)
end

function call_recv(data, clientid)
	local msg = clientid .. ": " .. data
	conn:send(msg)
	addline(msg)
end

function call_recv_client(data)	
	addline(data)
end

function addline(text)
	display = text .. "\n" .. display
end

function love.quit()
	if mode == "client" then
		conn:disconnect()
	end
end